unit bCRC32;

{32 Bit CRC, bitwise without table}


interface

(*************************************************************************

 DESCRIPTION     :  32 Bit CRC, bitwise without table

 REQUIREMENTS    :  D4-D7/D9-D10/D12, FPC

 EXTERNAL DATA   :  ---

 MEMORY USAGE    :  ---

 DISPLAY MODE    :  ---

 REFERENCES      :  ---


 Version  Date      Author      Modification
 -------  --------  -------     ------------------------------------------
 0.10     29.09.07  W.Ehrhardt  Initial version based on standard v3.03
 0.11     12.11.08  we          uses BTypes, Ptr2Inc and/or Str255/Str127
 0.12     19.07.09  we          D12 fix: assign with typecast string(fname)
**************************************************************************)

(*-------------------------------------------------------------------------
 (C) Copyright 2002-2009 Wolfgang Ehrhardt

 This software is provided 'as-is', without any express or implied warranty.
 In no event will the authors be held liable for any damages arising from
 the use of this software.

 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software in
    a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

 3. This notice may not be removed or altered from any source distribution.
----------------------------------------------------------------------------*)


{$i STD.INC}

uses
  BTypes;

procedure bCRC32Init(var CRC: longint);
  {-initialize context}

procedure bCRC32Update(var CRC: longint; Msg: pointer; Len: word);
  {-update CRC32 with Msg data}

procedure bCRC32Final(var CRC: longint);
  {-CRC32: finalize calculation}

function  bCRC32SelfTest: boolean;
  {-Self test for CRC32}

procedure bCRC32Full(var CRC: longint; Msg: pointer; Len: word);
  {-CRC32 of Msg with init/update/final}

procedure bCRC32File(const fname: Str255; var CRC: longint; var buf; bsize: word; var Err: word);
  {-CRC32 of file, buf: buffer with at least bsize bytes}

procedure bCRC32UpdateXL(var CRC: longint; Msg: pointer; Len: longint);
  {-update CRC32 with Msg data}

procedure bCRC32FullXL(var CRC: longint; Msg: pointer; Len: longint);
  {-CRC32 of Msg with init/update/final}


implementation


const
  Mask32 = $FFFFFFFF;
  CRC_PolyReflect: longint = longint($EDB88320);


{---------------------------------------------------------------------------}
procedure bCRC32Update(var CRC: longint; Msg: pointer; Len: word);
   {-update CRC32 with Msg data}
begin
  bCRC32UpdateXL(CRC,Msg,Len);
end;


{---------------------------------------------------------------------------}
procedure bCRC32UpdateXL(var CRC: longint; Msg: pointer; Len: longint);
  {-update CRC32 with Msg data}
var
  i,b: longint;
begin
  for i:=1 to Len do begin
    CRC := CRC xor PByte(Msg)^;
    for b := 1 to 8 do begin
      if odd(CRC) then CRC := (CRC shr 1) xor CRC_PolyReflect
      else CRC := CRC shr 1;
    end;
    inc(Ptr2Inc(Msg));
  end;
end;

{---------------------------------------------------------------------------}
procedure bCRC32FullXL(var CRC: longint; Msg: pointer; Len: longint);
  {-CRC32 of Msg with init/update/final}
begin
  bCRC32Init(CRC);
  bCRC32UpdateXL(CRC, Msg, Len);
  bCRC32Final(CRC);
end;


{---------------------------------------------------------------------------}
procedure bCRC32Init(var CRC: longint);
  {-CRC initialization}
begin
  CRC := longint(Mask32);
end;


{---------------------------------------------------------------------------}
procedure bCRC32Final(var CRC: longint);
  {-CRC32: finalize calculation}
begin
  CRC := CRC xor longint(Mask32);
end;



{---------------------------------------------------------------------------}
function  bCRC32SelfTest: boolean;
  {-self test for CRC32}
const
  s: string[3] = 'abc';
var
  i: integer;
  CRC, CRCF: longint;
begin
  bCRC32Full(CRCF, @s[1], length(s));
  bCRC32Init(CRC);
  for i:=1 to length(s) do bCRC32Update(CRC, @s[i], 1);
  bCRC32Final(CRC);
  bCRC32SelfTest := (CRC=$352441C2) and (CRCF=$352441C2);
end;


{---------------------------------------------------------------------------}
procedure bCRC32Full(var CRC: longint; Msg: pointer; Len: word);
  {-CRC32 of Msg with init/update/final}
begin
  bCRC32Init(CRC);
  bCRC32Update(CRC, Msg, Len);
  bCRC32Final(CRC);
end;


{$i-} {Force I-}
{---------------------------------------------------------------------------}
procedure bCRC32File(const fname: Str255; var CRC: longint; var buf; bsize: word; var Err: word);
  {-CRC32 of file, buf: buffer with at least bsize bytes}
var
  L: longint;
  fms: byte;
  f: file;
begin
  fms := FileMode;
  FileMode := 0;
  system.assign(f,{$ifdef D12Plus} string {$endif} (fname));
  system.reset(f,1);
  Err := IOResult;
  FileMode := fms;
  if Err<>0 then exit;
  bCRC32Init(CRC);
  L := bsize;
  while (Err=0) and (L=bsize) do begin
    system.blockread(f,buf,bsize,L);
    Err := IOResult;
    bCRC32UpdateXL(CRC, @buf, L);
  end;
  system.close(f);
  if IOResult=0 then;
  bCRC32Final(CRC);
end;


end.
